/*
  clipurl by Jeremy Weatherford

  clipboard-monitoring method from xcb by Farrell McKay
*/

/* change this as needed
   will be called like "%s 'url' &" */
#define BROWSER "mozilla"


#include <stdio.h>
#include <stdlib.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xatom.h>

#include <X11/Xmu/Atoms.h>
#include <X11/Xmu/StdSel.h>

#include <X11/Xaw/Command.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Cardinals.h>
#include <X11/Xfuncs.h>

#include <ctype.h>

Atom cutbuf, selection;

#define min(x,y) ((x) < (y) ? (x) : (y))

int debug = 0;

static void CBcb(Widget w, XtPointer data, Atom *selection, Atom *type,
		 XtPointer value, unsigned long *plen, int *format) {
  char buf[512], *p, *q;
  char cmdbuf[550], *c;
  int len = (int) *plen;
  int nl = 0;
  
  if (len > sizeof(buf)+1)
    len = sizeof(buf)-1;

  strncpy(buf, (char *)value, len);
  buf[len] = '\0';

  // skip initial whitespace and double-quotes
  for (p=buf; *p; p++,len--)
    if (*p != '"' && !isspace(*p)) break;

  if (debug)
    printf("CB contents: '%s'\n", buf);
  
  if (len < 7 || strncmp("http://", p, 7) != 0)
    return; // not a URL

  strcpy(cmdbuf, BROWSER);
  c = cmdbuf + strlen(cmdbuf);
  *c++ = ' ';
  *c++ = '\'';

  for (q=p; *q; q++) {
    // skip embedded newlines
    if (*q == '\n' || *q == '\r') { nl = 0; continue; } 
    if (isspace(*q)) { nl = 1; continue; }
    if (nl) break; // text after embedded space
    if (*q == '\'')
      *c++ = '\\';
    *c++ = *q;
  }

  // chop off trailing punctuation or double quotes
  *c-- = '\0';
  while (*c == '"' || *c == '.')
    *c-- = '\0';
  c++;

  *c++ = '\'';
  *c++ = ' ';
  *c++ = '&';
  *c++ = '\0';

  // full cmdline: "BROWSER 'url' &"

  if (debug)
    printf("%s\n", cmdbuf);
  system(cmdbuf);
}


static void grabCB(Widget w, XtPointer closure, XtPointer callData) {
  XtGetSelectionValue(w, selection, XA_STRING, CBcb, NULL, 
		      XtLastTimestampProcessed(XtDisplay(w)));
}

int main(int argc, char *argv[]) {
  Widget win, box, button;
  Display *dpy;
  String selname = "PRIMARY";
  XtAppContext appcon;
  XEvent ev;
  char **p;

  XtSetLanguageProc(NULL, NULL, NULL);

  win = XtAppInitialize(&appcon, "clipurl", NULL, 0,
			&argc, argv, NULL, NULL, 0);

  for (p = argv + 1; p < argv + argc; p++) {
    if (!strcmp(*p, "-d"))
      debug++;
  }

  box = XtCreateManagedWidget("box", boxWidgetClass, win, NULL, ZERO);

  button =
    XtCreateManagedWidget("clipurl", commandWidgetClass, box, NULL, ZERO);
  XtAddCallback( button, XtNcallback, grabCB, NULL );

  dpy = XtDisplay(win);

  // we want PropertyNotify events from the root window
  XSelectInput(dpy, RootWindow(dpy, DefaultScreen(dpy)), PropertyChangeMask);

  // not entirely sure why these are different
  // we monitor cutbuf at the root window, but get the contents of selection
  XmuInternStrings(XtDisplay(win), &selname, ONE, &selection);
  cutbuf = XA_CUT_BUFFER0;

  XtRealizeWidget(win);

  while (1) {
    XtAppNextEvent(appcon, &ev);

    if (ev.type == PropertyNotify) {
      if (ev.xproperty.atom == cutbuf)
	// selection changed
	grabCB(button, NULL, NULL);
    } else
      XtDispatchEvent(&ev);
  }
}
